#include "utils/directories.hpp"
#include <iostream>
#include <fstream>
#include <boost/filesystem.hpp>
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/ini_parser.hpp>
#include "config.hpp"
#include "utils/envmanager.hpp"

namespace fs = boost::filesystem;
namespace pt = boost::property_tree;

namespace dropshell {


config *get_global_config() {
    static config *gConfig = new config();
    return gConfig;
}

config::config() {
}
config::~config() {
}

bool config::load_config() {
    std::string config_path = get_local_dropshell_config_path();
    if (config_path.empty())
        return false;

    envmanager config_env(config_path);
    if (!config_env.load())
        {
            std::cerr << "Warning: Unable to read configuration file: " << config_path << std::endl;
            return false;
        }

    local_config_directory = config_env.get_variable_substituted("local.config.directory");
    if (local_config_directory.empty())
        {
            std::cerr << "Warning: User directory not set in config" << std::endl;
            return false;
        }
    return true;
}

void config::save_config()
{
    std::string config_path = get_local_dropshell_config_path();
    if (config_path.empty())
    {
        std::cerr << "Warning: Unable to save configuration file, as DropShell has not been initialised."<< std::endl;
        std::cerr << "Please run 'dropshell init <path>' to initialise DropShell." << std::endl;
        return;
    }

    envmanager config_env(config_path);
    config_env.set_variable("local.config.directory", local_config_directory);
    config_env.save();
}

bool config::is_config_set() const
{
    return !local_config_directory.empty();
}

bool config::get_local_config_directory(std::string& path) const {
    path = local_config_directory;
    return !path.empty();
}

void config::init_local_config_directory(const std::string& path) {
    // Convert to canonical path
    fs::path abs_path = fs::canonical(path);
    
    // The directory must exist
    if (!fs::exists(abs_path)) {
        throw std::runtime_error("The specifieduser directory does not exist: " + abs_path.string());
    }

    local_config_directory = abs_path.string();
    save_config();
    std::cout << "Local config directory initialized to: " << abs_path.string() << std::endl;
}


} // namespace dropshell 